unit becon;

{$S-,W-,R-}
{$C PRELOAD}

interface

uses Messages, Windows, SysUtils, Classes, Controls, Forms, Menus, Graphics,
  StdCtrls;

Type
  TCBevelStyle = (bsLowered, bsRaised);
  TCBevelShape = (bsBox,
                  bsFrame,
                  bsTopLine,
                  bsBottomLine,
                  bsLeftLine,
                  bsRightLine,
                  bsLRLine,
                  bsTBLine,
                  bsTLLine,
                  bsTRLine,
                  bsBLLine,
                  bsBRLine,
                  bsTLRLine,
                  bsBLRLine);

  TCBevel = class(TGraphicControl)
  private
    FStyle: TCBevelStyle;
    FShape: TCBevelShape;
    procedure SetStyle(Value: TCBevelStyle);
    procedure SetShape(Value: TCBevelShape);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    property ParentShowHint;
    property Shape: TCBevelShape read FShape write SetShape default bsBox;
    property ShowHint;
    property Style: TCBevelStyle read FStyle write SetStyle default bsLowered;
    property Visible;
  end;

procedure Register;

Implementation

{ TCBevel }

constructor TCBevel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FStyle := bsLowered;
  FShape := bsBox;
  Width := 50;
  Height := 50;
end;

procedure TCBevel.SetStyle(Value: TCBevelStyle);
begin
  if Value <> FStyle then
  begin
    FStyle := Value;
    Invalidate;
  end;
end;

procedure TCBevel.SetShape(Value: TCBevelShape);
begin
  if Value <> FShape then
  begin
    FShape := Value;
    Invalidate;
  end;
end;

procedure TCBevel.Paint;
var
  Color1, Color2: TColor;
  Temp: TColor;

  procedure CBevelRect(const R: TRect);
  begin
    with Canvas do
    begin
      Pen.Color := Color1;
      PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
        Point(R.Right, R.Top)]);
      Pen.Color := Color2;
      PolyLine([Point(R.Right, R.Top), Point(R.Right, R.Bottom),
        Point(R.Left, R.Bottom)]);
    end;
  end;

  procedure CBevelLine(C: TColor; X1, Y1, X2, Y2: Integer);
  begin
    with Canvas do
    begin
      Pen.Color := C;
      MoveTo(X1, Y1);
      LineTo(X2, Y2);
    end;
  end;

begin
  with Canvas do
  begin
    Pen.Width := 1;

    if FStyle = bsLowered then
    begin
      Color1 := clBtnShadow;
      Color2 := clBtnHighlight;
    end
    else
    begin
      Color1 := clBtnHighlight;
      Color2 := clBtnShadow;
    end;

    case FShape of
      bsBox: CBevelRect(Rect(0, 0, Width - 1, Height - 1));
      bsFrame:
        begin
          Temp := Color1;
          Color1 := Color2;
          CBevelRect(Rect(1, 1, Width - 1, Height - 1));
          Color2 := Temp;
          Color1 := Temp;
          CBevelRect(Rect(0, 0, Width - 2, Height - 2));
        end;
      bsTopLine:
        begin
          CBevelLine(Color1, 0, 0, Width, 0);
          CBevelLine(Color2, 0, 1, Width, 1);
        end;
      bsBottomLine:
        begin
          CBevelLine(Color1, 0, Height - 2, Width, Height - 2);
          CBevelLine(Color2, 0, Height - 1, Width, Height - 1);
        end;
      bsLeftLine:
        begin
          CBevelLine(Color1, 0, 0, 0, Height);
          CBevelLine(Color2, 1, 0, 1, Height);
        end;
      bsRightLine:
        begin
          CBevelLine(Color1, Width - 2, 0, Width - 2, Height);
          CBevelLine(Color2, Width - 1, 0, Width - 1, Height);
        end;
      bsLRLine:
        begin
          CBevelLine(Color1, 0, 0, 0, Height);
          CBevelLine(Color2, 1, 0, 1, Height);
          CBevelLine(Color1, Width - 2, 0, Width - 2, Height);
          CBevelLine(Color2, Width - 1, 0, Width - 1, Height);
        end;
      bsTBLine:
        begin
          CBevelLine(Color1, 0, 0, Width, 0);
          CBevelLine(Color2, 0, 1, Width, 1);
          CBevelLine(Color1, 0, Height - 2, Width, Height - 2);
          CBevelLine(Color2, 0, Height - 1, Width, Height - 1);
        end;
      bsTLLine:
        begin
          CBevelLine(Color1, 0, 0, Width, 0);
          CBevelLine(Color2, 0, 1, Width, 1);
          CBevelLine(Color1, 0, 0, 0, Height);
          CBevelLine(Color2, 1, 0, 1, Height);
        end;
      bsTRLine:
        begin
          CBevelLine(Color1, 0, 0, Width, 0);
          CBevelLine(Color2, 0, 1, Width, 1);
          CBevelLine(Color1, Width - 2, 0, Width - 2, Height);
          CBevelLine(Color2, Width - 1, 0, Width - 1, Height);
        end;
      bsBLLine:
        begin
          CBevelLine(Color1, 0, Height - 2, Width, Height - 2);
          CBevelLine(Color2, 0, Height - 1, Width, Height - 1);
          CBevelLine(Color1, 0, 0, 0, Height);
          CBevelLine(Color2, 1, 0, 1, Height);
        end;
      bsBRLine:
        begin
          CBevelLine(Color1, 0, Height - 2, Width, Height - 2);
          CBevelLine(Color2, 0, Height - 1, Width, Height - 1);
          CBevelLine(Color1, Width - 2, 0, Width - 2, Height);
          CBevelLine(Color2, Width - 1, 0, Width - 1, Height);
        end;
      bsTLRLine:
        begin
          CBevelLine(Color1, 0, 0, Width, 0);
          CBevelLine(Color2, 0, 1, Width, 1);
          CBevelLine(Color1, 0, 0, 0, Height);
          CBevelLine(Color2, 1, 0, 1, Height);
          CBevelLine(Color1, Width - 2, 0, Width - 2, Height);
          CBevelLine(Color2, Width - 1, 0, Width - 1, Height);
        end;
      bsBLRLine:
        begin
          CBevelLine(Color1, 0, 0, 0, Height);
          CBevelLine(Color2, 1, 0, 1, Height);
          CBevelLine(Color1, Width - 2, 0, Width - 2, Height);
          CBevelLine(Color2, Width - 1, 0, Width - 1, Height);
          CBevelLine(Color1, 0, Height - 2, Width, Height - 2);
          CBevelLine(Color2, 0, Height - 1, Width, Height - 1);
        end;
    end;
  end;
end;

procedure Register;
begin
    RegisterComponents('Additional', [TCBevel]);
end;

end.
